@@ -39,15 +39,15 @@ _SCHEMA_JSON = {
         "grandparent": {
             "join": "parent",
             "joinOn": "grandparent.id = parent.grandparent_id",
-            "name": "grandparent"
+            "name": "grandparent",
         },
         "parent": {
+            "key": [{"name": "id"}],
             "join": "child",
             "joinKey": ["id"],
             "joinMode": "async",
             "joinOn": "parent.id = child.parent_id",
-            "key": ["id"],
-            "name": "parent"
+            "name": "parent",
         },
     },
     "targetTable": {
@@ -58,7 +58,7 @@ _SCHEMA_JSON = {
     },
     "targetQuery": """
         SELECT c.id, g.name, p.name
-        FROM $1 AS d
+        FROM ${key} AS d
             JOIN child c ON d.id = c.id
             JOIN parent p ON c.parent_id = p.id
             JOIN grandparent AS g ON p.grandparent_id = g.id
